import json
from functools import wraps
from loguru import logger


class MsGraphException(Exception):
    pass


class AcquireTokenException(MsGraphException):
    """Raised when can`t get token"""


class InvalidRequestException(MsGraphException):
    pass


class ClientCredentialsError(MsGraphException):
    """
    Raised when wrong client id or secret passed
    """


class ItemNotFoundException(MsGraphException):
    """
    Raised when item not found
    """


class UnrecognizedNotFoundException(MsGraphException):
    pass


class BadRequestException(MsGraphException):
    pass


class InvalidSessionException(MsGraphException):
    pass


class InvalidAuthenticationTokenExceltion(MsGraphException):
    pass


class UnrecognizedBadRequestException(MsGraphException):
    pass


class UnrecognizedAuthorizationException(MsGraphException):
    pass


class InvalidArgumentException(MsGraphException):
    pass


class UnrecognizedHTTPError(MsGraphException):
    pass


MAP_MSGRAPH_ERROR_TO_EXCEPTIONS = {
    400: {
        "invalidrequest": InvalidRequestException,
        "badrequest": BadRequestException,
        "invalidsession": InvalidSessionException,
        "invalidargument": InvalidArgumentException,
        "default": UnrecognizedBadRequestException,
    },
    401: {
        "invalidauthenticationtoken": InvalidAuthenticationTokenExceltion,
        "default": UnrecognizedAuthorizationException,
    },
    404: {
        "itemnotfound": ItemNotFoundException,
        "default": UnrecognizedNotFoundException,
    }
}


def get_exception(http_code: int, error_code: str, data: str):
    codes = MAP_MSGRAPH_ERROR_TO_EXCEPTIONS.get(http_code)
    if codes:
        if error_code and error_code.lower() in codes:
            exception = codes[error_code.lower()]
        else:
            exception = codes['default']
        return exception(data)
    else:
        return UnrecognizedHTTPError(data)


def handle_http_errors(response):
    if response.status_code >= 400:
        error = response.json()
        error_code = error.get('error', {}).get(
            'code') if error else "default"
        exception = get_exception(
            response.status_code, error_code, json.dumps(error))
        logger.exception(exception)
        raise exception

    return response
